@@ -30,11 +30,14 @@
 
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file.h>
-#include <eel/eel-glib-extensions.h>
-#include <eel/eel-string.h>
+#include <libnautilus-private/nautilus-icon-names.h>
+
 #include <gio/gio.h>
+#include <string.h>
 
 #define MAX_BOOKMARK_LENGTH 80
+#define LOAD_JOB 1
+#define SAVE_JOB 2
 
 enum {
 	CONTENTS_CHANGED,
@@ -43,14 +46,14 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 static char *window_geometry;
+static NautilusBookmarkList *singleton = NULL;
 
 /* forward declarations */
 
-static void        destroy                              (GtkObject            *object);
 static void        nautilus_bookmark_list_load_file     (NautilusBookmarkList *bookmarks);
 static void        nautilus_bookmark_list_save_file     (NautilusBookmarkList *bookmarks);
 
-G_DEFINE_TYPE(NautilusBookmarkList, nautilus_bookmark_list, GTK_TYPE_OBJECT)
+G_DEFINE_TYPE(NautilusBookmarkList, nautilus_bookmark_list, G_TYPE_OBJECT)
 
 static NautilusBookmark *
 new_bookmark_from_uri (const char *uri, const char *label)
@@ -61,6 +64,7 @@ new_bookmark_from_uri (const char *uri, const char *label)
 	GIcon *icon;
 	gboolean has_label;
 	GFile *location;
+	gboolean native;
 
 	location = NULL;
 	if (uri) {
@@ -78,8 +82,9 @@ new_bookmark_from_uri (const char *uri, const char *label)
 	new_bookmark = NULL;
 	
 	if (uri) {
+		native = g_file_is_native (location);
 		file = nautilus_file_get (location);
-		
+
 		icon = NULL;
 		if (nautilus_file_check_if_ready (file,
 						  NAUTILUS_FILE_ATTRIBUTES_FOR_ICON)) {
@@ -88,10 +93,11 @@ new_bookmark_from_uri (const char *uri, const char *label)
 		nautilus_file_unref (file);
 		
 		if (icon == NULL) {
-			icon = g_themed_icon_new ("gnome-fs-directory");
+			icon = native ? g_themed_icon_new (NAUTILUS_ICON_FOLDER) :
+				g_themed_icon_new (NAUTILUS_ICON_FOLDER_REMOTE);
 		}
 
-		new_bookmark = nautilus_bookmark_new_with_icon (location, name, has_label, icon);
+		new_bookmark = nautilus_bookmark_new (location, name, has_label, icon);
 
 		g_object_unref (icon);
 
@@ -119,56 +125,6 @@ nautilus_bookmark_list_get_file (void)
 
 /* Initialization.  */
 
-static void
-nautilus_bookmark_list_class_init (NautilusBookmarkListClass *class)
-{
-	GtkObjectClass *object_class;
-
-	object_class = GTK_OBJECT_CLASS (class);
-
-	object_class->destroy = destroy;
-
-	signals[CONTENTS_CHANGED] =
-		g_signal_new ("contents_changed",
-		              G_TYPE_FROM_CLASS (object_class),
-		              G_SIGNAL_RUN_LAST,
-		              G_STRUCT_OFFSET (NautilusBookmarkListClass, 
-						   contents_changed),
-		              NULL, NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE, 0);
-}
-
-static void
-bookmark_monitor_changed_cb (GFileMonitor      *monitor,
-			     GFile             *child,
-			     GFile             *other_file,
-			     GFileMonitorEvent  eflags,
-			     gpointer           user_data)
-{
-	if (eflags == G_FILE_MONITOR_EVENT_CHANGED ||
-	    eflags == G_FILE_MONITOR_EVENT_CREATED) {
-		g_return_if_fail (NAUTILUS_IS_BOOKMARK_LIST (NAUTILUS_BOOKMARK_LIST (user_data)));
-		nautilus_bookmark_list_load_file (NAUTILUS_BOOKMARK_LIST (user_data));
-		g_signal_emit (user_data, signals[CONTENTS_CHANGED], 0);
-	}
-}
-
-static void
-nautilus_bookmark_list_init (NautilusBookmarkList *bookmarks)
-{
-	GFile *file;
-
-	nautilus_bookmark_list_load_file (bookmarks);
-
-	file = nautilus_bookmark_list_get_file ();
-	bookmarks->monitor = g_file_monitor_file (file, 0, NULL, NULL);
-	g_signal_connect (bookmarks->monitor, "changed",
-			  G_CALLBACK (bookmark_monitor_changed_cb), bookmarks);
-
-	g_object_unref (file);
-}
-
 static void
 bookmark_in_list_changed_callback (NautilusBookmark     *bookmark,
 				   NautilusBookmarkList *bookmarks)
@@ -177,7 +133,7 @@ bookmark_in_list_changed_callback (NautilusBookmark     *bookmark,
 	g_assert (NAUTILUS_IS_BOOKMARK_LIST (bookmarks));
 
 	/* Save changes so we'll have the good icon next time. */
-	nautilus_bookmark_list_contents_changed (bookmarks);
+	nautilus_bookmark_list_save_file (bookmarks);
 }
 
 static void
@@ -203,19 +159,96 @@ static void
 clear (NautilusBookmarkList *bookmarks)
 {
 	g_list_foreach (bookmarks->list, stop_monitoring_one, bookmarks);
-	eel_g_object_list_free (bookmarks->list);
+	g_list_free_full (bookmarks->list, g_object_unref);
 	bookmarks->list = NULL;
 }
 
 static void
-destroy (GtkObject *object)
+do_finalize (GObject *object)
 {
 	if (NAUTILUS_BOOKMARK_LIST (object)->monitor != NULL) {
 		g_file_monitor_cancel (NAUTILUS_BOOKMARK_LIST (object)->monitor);
 		NAUTILUS_BOOKMARK_LIST (object)->monitor = NULL;
 	}
 
+	g_queue_free (NAUTILUS_BOOKMARK_LIST (object)->pending_ops);
+
 	clear (NAUTILUS_BOOKMARK_LIST (object));
+
+	G_OBJECT_CLASS (nautilus_bookmark_list_parent_class)->finalize (object);
+}
+
+static GObject *
+do_constructor (GType type,
+                guint n_construct_params,
+                GObjectConstructParam *construct_params)
+{
+	GObject *retval;
+
+	if (singleton != NULL) {
+		return g_object_ref (singleton);
+	}
+
+	retval = G_OBJECT_CLASS (nautilus_bookmark_list_parent_class)->constructor
+		(type, n_construct_params, construct_params);
+
+	singleton = NAUTILUS_BOOKMARK_LIST (retval);
+	g_object_add_weak_pointer (retval, (gpointer) &singleton);
+
+	return retval;
+}
+
+
+static void
+nautilus_bookmark_list_class_init (NautilusBookmarkListClass *class)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+	object_class->finalize = do_finalize;
+	object_class->constructor = do_constructor;
+
+	signals[CONTENTS_CHANGED] =
+		g_signal_new ("contents_changed",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (NautilusBookmarkListClass, 
+						   contents_changed),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE, 0);
+}
+
+static void
+bookmark_monitor_changed_cb (GFileMonitor      *monitor,
+			     GFile             *child,
+			     GFile             *other_file,
+			     GFileMonitorEvent  eflags,
+			     gpointer           user_data)
+{
+	if (eflags == G_FILE_MONITOR_EVENT_CHANGED ||
+	    eflags == G_FILE_MONITOR_EVENT_CREATED) {
+		g_return_if_fail (NAUTILUS_IS_BOOKMARK_LIST (NAUTILUS_BOOKMARK_LIST (user_data)));
+		nautilus_bookmark_list_load_file (NAUTILUS_BOOKMARK_LIST (user_data));
+	}
+}
+
+static void
+nautilus_bookmark_list_init (NautilusBookmarkList *bookmarks)
+{
+	GFile *file;
+
+	bookmarks->pending_ops = g_queue_new ();
+
+	nautilus_bookmark_list_load_file (bookmarks);
+
+	file = nautilus_bookmark_list_get_file ();
+	bookmarks->monitor = g_file_monitor_file (file, 0, NULL, NULL);
+	g_file_monitor_set_rate_limit (bookmarks->monitor, 1000);
+
+	g_signal_connect (bookmarks->monitor, "changed",
+			  G_CALLBACK (bookmark_monitor_changed_cb), bookmarks);
+
+	g_object_unref (file);
 }
 
 static void
@@ -225,8 +258,6 @@ insert_bookmark_internal (NautilusBookmarkList *bookmarks,
 {
 	bookmarks->list = g_list_insert (bookmarks->list, bookmark, index);
 
-	g_signal_connect_object (bookmark, "appearance_changed",
-				 G_CALLBACK (bookmark_in_list_changed_callback), bookmarks, 0);
 	g_signal_connect_object (bookmark, "contents_changed",
 				 G_CALLBACK (bookmark_in_list_changed_callback), bookmarks, 0);
 }
@@ -249,7 +280,7 @@ nautilus_bookmark_list_append (NautilusBookmarkList *bookmarks,
 				  nautilus_bookmark_copy (bookmark), 
 				  -1);
 
-	nautilus_bookmark_list_contents_changed (bookmarks);
+	nautilus_bookmark_list_save_file (bookmarks);
 }
 
 /**
@@ -274,21 +305,6 @@ nautilus_bookmark_list_contains (NautilusBookmarkList *bookmarks,
 		!= NULL;
 }
 
-/**
- * nautilus_bookmark_list_contents_changed:
- * 
- * Save the bookmark list to disk, and emit the contents_changed signal.
- * @bookmarks: NautilusBookmarkList whose contents have been modified.
- **/ 
-void
-nautilus_bookmark_list_contents_changed (NautilusBookmarkList *bookmarks)
-{
-	g_return_if_fail (NAUTILUS_IS_BOOKMARK_LIST (bookmarks));
-
-	nautilus_bookmark_list_save_file (bookmarks);
-	g_signal_emit (bookmarks, signals[CONTENTS_CHANGED], 0);
-}
-
 /**
  * nautilus_bookmark_list_delete_item_at:
  * 
@@ -314,7 +330,42 @@ nautilus_bookmark_list_delete_item_at (NautilusBookmarkList *bookmarks,
 
 	g_list_free_1 (doomed);
 
-	nautilus_bookmark_list_contents_changed (bookmarks);
+	nautilus_bookmark_list_save_file (bookmarks);
+}
+
+/**
+ * nautilus_bookmark_list_move_item:
+ *
+ * Move the item from the given position to the destination.
+ * @index: the index of the first bookmark.
+ * @destination: the index of the second bookmark.
+ **/
+void
+nautilus_bookmark_list_move_item (NautilusBookmarkList *bookmarks,
+				  guint index,
+				  guint destination)
+{
+	GList *bookmark_item;
+
+	if (index == destination) {
+		return;
+	}
+
+	bookmark_item = g_list_nth (bookmarks->list, index);
+	bookmarks->list = g_list_remove_link (bookmarks->list,
+					      bookmark_item);
+
+	if (index < destination) {
+		bookmarks->list = g_list_insert (bookmarks->list,
+						 bookmark_item->data,
+						 destination - 1);
+	} else {
+		bookmarks->list = g_list_insert (bookmarks->list,
+						 bookmark_item->data,
+						 destination);
+	}
+
+	nautilus_bookmark_list_save_file (bookmarks);
 }
 
 /**
@@ -340,7 +391,7 @@ nautilus_bookmark_list_delete_items_with_uri (NautilusBookmarkList *bookmarks,
 		next = node->next;
 
 		bookmark_uri = nautilus_bookmark_get_uri (NAUTILUS_BOOKMARK (node->data));
-		if (eel_strcmp (bookmark_uri, uri) == 0) {
+		if (g_strcmp0 (bookmark_uri, uri) == 0) {
 			bookmarks->list = g_list_remove_link (bookmarks->list, node);
 			stop_monitoring_bookmark (bookmarks, NAUTILUS_BOOKMARK (node->data));
 			g_object_unref (node->data);
@@ -351,7 +402,7 @@ nautilus_bookmark_list_delete_items_with_uri (NautilusBookmarkList *bookmarks,
 	}
 
 	if (list_changed) {
-		nautilus_bookmark_list_contents_changed (bookmarks);
+		nautilus_bookmark_list_save_file (bookmarks);
 	}
 }
 
@@ -391,7 +442,7 @@ nautilus_bookmark_list_insert_item (NautilusBookmarkList *bookmarks,
 				  nautilus_bookmark_copy (new_bookmark), 
 				  index);
 
-	nautilus_bookmark_list_contents_changed (bookmarks);
+	nautilus_bookmark_list_save_file (bookmarks);
 }
 
 /**
@@ -428,31 +479,25 @@ nautilus_bookmark_list_length (NautilusBookmarkList *bookmarks)
 	return g_list_length (bookmarks->list);
 }
 
-/**
- * nautilus_bookmark_list_load_file:
- * 
- * Reads bookmarks from file, clobbering contents in memory.
- * @bookmarks: the list of bookmarks to fill with file contents.
- **/
 static void
-nautilus_bookmark_list_load_file (NautilusBookmarkList *bookmarks)
+load_file_finish (NautilusBookmarkList *bookmarks,
+		  GObject *source,
+		  GAsyncResult *res)
 {
-	GFile *file;
-	char *contents;
 	GError *error = NULL;
+	gchar *contents = NULL;
 
-	file = nautilus_bookmark_list_get_file ();
+	g_file_load_contents_finish (G_FILE (source),
+				     res, &contents, NULL, NULL, &error);
 
-	/* Wipe out old list. */
-	clear (bookmarks);
-
-	if (g_file_load_contents (file, NULL, &contents, NULL, NULL, &error)) {
+	if (error == NULL) {
         	char **lines;
       		int i;
 
 		lines = g_strsplit (contents, "\n", -1);
       	 	for (i = 0; lines[i]; i++) {
-	  		if (lines[i][0]) {
+			/* Ignore empty or invalid lines that cannot be parsed properly */
+	  		if (lines[i][0] != '\0' && lines[i][0] != ' ') {
 				/* gtk 2.7/2.8 might have labels appended to bookmarks which are separated by a space */
 				/* we must seperate the bookmark uri and the potential label */
  				char *space, *label;
@@ -472,15 +517,184 @@ nautilus_bookmark_list_load_file (NautilusBookmarkList *bookmarks)
 		}
       		g_free (contents);
        		g_strfreev (lines);
-	}
-	else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
+
+		g_signal_emit (bookmarks, signals[CONTENTS_CHANGED], 0);
+	} else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
 		g_warning ("Could not load bookmark file: %s\n", error->message);
 		g_error_free (error);
 	}
+}
+
+static void
+load_file_async (NautilusBookmarkList *self,
+		 GAsyncReadyCallback callback)
+{
+	GFile *file;
+
+	file = nautilus_bookmark_list_get_file ();
+
+	/* Wipe out old list. */
+	clear (self);
+
+	/* keep the bookmark list alive */
+	g_object_ref (self);
+	g_file_load_contents_async (file, NULL, callback, self);
 
 	g_object_unref (file);
 }
 
+static void
+save_file_finish (NautilusBookmarkList *bookmarks,
+		  GObject *source,
+		  GAsyncResult *res)
+{
+	GError *error = NULL;
+	GFile *file;
+
+	g_file_replace_contents_finish (G_FILE (source),
+					res, NULL, &error);
+
+	if (error != NULL) {
+		g_warning ("Unable to replace contents of the bookmarks file: %s",
+			   error->message);
+		g_error_free (error);
+	}
+
+	file = nautilus_bookmark_list_get_file ();
+
+	/* re-enable bookmark file monitoring */
+	bookmarks->monitor = g_file_monitor_file (file, 0, NULL, NULL);
+	g_file_monitor_set_rate_limit (bookmarks->monitor, 1000);
+	g_signal_connect (bookmarks->monitor, "changed",
+			  G_CALLBACK (bookmark_monitor_changed_cb), bookmarks);
+
+	g_object_unref (file);
+}
+
+static void
+save_file_async (NautilusBookmarkList *bookmarks,
+		 GAsyncReadyCallback callback)
+{
+	GFile *file;
+	GList *l;
+	GString *bookmark_string;
+
+	/* temporarily disable bookmark file monitoring when writing file */
+	if (bookmarks->monitor != NULL) {
+		g_file_monitor_cancel (bookmarks->monitor);
+		bookmarks->monitor = NULL;
+	}
+
+	file = nautilus_bookmark_list_get_file ();
+	bookmark_string = g_string_new (NULL);
+
+	for (l = bookmarks->list; l; l = l->next) {
+		NautilusBookmark *bookmark;
+
+		bookmark = NAUTILUS_BOOKMARK (l->data);
+
+		/* make sure we save label if it has one for compatibility with GTK 2.7 and 2.8 */
+		if (nautilus_bookmark_get_has_custom_name (bookmark)) {
+			char *label, *uri;
+			label = nautilus_bookmark_get_name (bookmark);
+			uri = nautilus_bookmark_get_uri (bookmark);
+			g_string_append_printf (bookmark_string,
+						"%s %s\n", uri, label);
+			g_free (uri);
+			g_free (label);
+		} else {
+			char *uri;
+			uri = nautilus_bookmark_get_uri (bookmark);
+			g_string_append_printf (bookmark_string, "%s\n", uri);
+			g_free (uri);
+		}
+	}
+
+	/* keep the bookmark list alive */
+	g_object_ref (bookmarks);
+	g_file_replace_contents_async (file, bookmark_string->str,
+				       bookmark_string->len, NULL,
+				       FALSE, 0, NULL, callback,
+				       bookmarks);
+
+	g_object_unref (file);
+}
+
+static void
+process_next_op (NautilusBookmarkList *bookmarks);
+
+static void
+op_processed_cb (GObject *source,
+		 GAsyncResult *res,
+		 gpointer user_data)
+{
+	NautilusBookmarkList *self = user_data;
+	int op;
+
+	op = GPOINTER_TO_INT (g_queue_pop_tail (self->pending_ops));
+
+	if (op == LOAD_JOB) {
+		load_file_finish (self, source, res);
+	} else {
+		save_file_finish (self, source, res);
+	}
+
+	if (!g_queue_is_empty (self->pending_ops)) {
+		process_next_op (self);
+	}
+
+	/* release the reference acquired during the _async method */
+	g_object_unref (self);
+}
+
+static void
+process_next_op (NautilusBookmarkList *bookmarks)
+{
+	gint op;
+
+	op = GPOINTER_TO_INT (g_queue_peek_tail (bookmarks->pending_ops));
+
+	if (op == LOAD_JOB) {
+		load_file_async (bookmarks, op_processed_cb);
+	} else {
+		save_file_async (bookmarks, op_processed_cb);
+	}
+}
+
+/**
+ * nautilus_bookmark_list_load_file:
+ * 
+ * Reads bookmarks from file, clobbering contents in memory.
+ * @bookmarks: the list of bookmarks to fill with file contents.
+ **/
+static void
+nautilus_bookmark_list_load_file (NautilusBookmarkList *bookmarks)
+{
+	g_queue_push_head (bookmarks->pending_ops, GINT_TO_POINTER (LOAD_JOB));
+
+	if (g_queue_get_length (bookmarks->pending_ops) == 1) {
+		process_next_op (bookmarks);
+	}
+}
+
+/**
+ * nautilus_bookmark_list_save_file:
+ * 
+ * Save bookmarks to disk.
+ * @bookmarks: the list of bookmarks to save.
+ **/
+static void
+nautilus_bookmark_list_save_file (NautilusBookmarkList *bookmarks)
+{
+	g_signal_emit (bookmarks, signals[CONTENTS_CHANGED], 0);
+
+	g_queue_push_head (bookmarks->pending_ops, GINT_TO_POINTER (SAVE_JOB));
+
+	if (g_queue_get_length (bookmarks->pending_ops) == 1) {
+		process_next_op (bookmarks);
+	}
+}
+
 /**
  * nautilus_bookmark_list_new:
  * 
@@ -495,88 +709,7 @@ nautilus_bookmark_list_new (void)
 
 	list = NAUTILUS_BOOKMARK_LIST (g_object_new (NAUTILUS_TYPE_BOOKMARK_LIST, NULL));
 
-	return g_object_ref_sink (list);
-}
-
-/**
- * nautilus_bookmark_list_save_file:
- * 
- * Save bookmarks to disk.
- * @bookmarks: the list of bookmarks to save.
- **/
-static void
-nautilus_bookmark_list_save_file (NautilusBookmarkList *bookmarks)
-{
-	GFile *file;
-	GOutputStream *out;
-	GError *error = NULL;
-	GList *l;
-
-	/* temporarily disable bookmark file monitoring when writing file */
-	if (bookmarks->monitor != NULL) {
-		g_file_monitor_cancel (bookmarks->monitor);
-		bookmarks->monitor = NULL;
-	}
-
-	file = nautilus_bookmark_list_get_file ();
-
-	out = (GOutputStream *)g_file_replace (file, NULL, FALSE, 0, NULL, &error);
-	if (out == NULL) {
-		g_warning ("Error opening bookmark file: %s\n", error->message);
-		goto error;
-	}
-
-      	for (l = bookmarks->list; l; l = l->next) {
-		NautilusBookmark *bookmark;
-		char *bookmark_string;
-
-		bookmark = NAUTILUS_BOOKMARK (l->data);
-
-		/* make sure we save label if it has one for compatibility with GTK 2.7 and 2.8 */
-		if (nautilus_bookmark_get_has_custom_name (bookmark)) {
-			char *label, *uri;
-			label = nautilus_bookmark_get_name (bookmark);
-			uri = nautilus_bookmark_get_uri (bookmark);
-			bookmark_string = g_strconcat (uri, " ", label, "\n", NULL);
-			g_free (uri);
-			g_free (label);
-		} else {
-			char *uri;
-			uri = nautilus_bookmark_get_uri (bookmark);
-			bookmark_string = g_strconcat (uri, "\n", NULL);
-			g_free (uri);
-		}
-
-		if (!g_output_stream_write_all (out,
-						bookmark_string,
-						strlen (bookmark_string),
-						NULL, NULL,
-						&error)) {
-			g_warning ("writing %s to bookmark file failed: %s\n",
-				   bookmark_string, error->message); 
-			g_free (bookmark_string);
-    			goto error;
-		}
-
-		g_free (bookmark_string);
-	}
-
-	if (!g_output_stream_close (out, NULL, &error)) {
-		g_warning ("Error closing bookmark file: %s\n", error->message);
-	}
-
-error:
-	if (error)
-		g_error_free (error);
-
-	/* re-enable bookmark file monitoring */
-	bookmarks->monitor = g_file_monitor_file (file, 0, NULL, NULL);
-	g_signal_connect (bookmarks->monitor, "changed",
-			  G_CALLBACK (bookmark_monitor_changed_cb), bookmarks);
-
-	if (out)
-		g_object_unref (out);
-	g_object_unref (file);
+	return list;
 }
 
 /**
