@@ -343,7 +343,7 @@ finalize (GObject *object)
 }
 
 static void
-destroy (GtkObject *object)
+destroy (GtkWidget *object)
 {
 	NautilusLocationBar *bar;
 
@@ -358,21 +358,21 @@ destroy (GtkObject *object)
 	g_free (bar->details->last_location);
 	bar->details->last_location = NULL;
 	
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (GTK_WIDGET_CLASS, destroy, (object));
 }
 
 static void
 nautilus_location_bar_class_init (NautilusLocationBarClass *class)
 {
 	GObjectClass *gobject_class;
-	GtkObjectClass *object_class;
+	GtkWidgetClass *widget_class;
 	NautilusNavigationBarClass *navigation_bar_class;
 
 	gobject_class = G_OBJECT_CLASS (class);
 	gobject_class->finalize = finalize;
-	
-	object_class = GTK_OBJECT_CLASS (class);
-	object_class->destroy = destroy;
+
+	widget_class = GTK_WIDGET_CLASS (class);
+	widget_class->destroy = destroy;
 	
 	navigation_bar_class = NAUTILUS_NAVIGATION_BAR_CLASS (class);
 
@@ -573,16 +573,16 @@ nautilus_location_bar_update_label (NautilusLocationBar *bar)
 void
 nautilus_location_bar_set_active(NautilusLocationBar *location_bar, gboolean is_active)
 {
-	if(is_active) {
+	if (is_active) {
 		/* reset style to default */
-		gtk_widget_modify_base (GTK_WIDGET (location_bar->details->entry), GTK_STATE_NORMAL, NULL);
-	}
-	else {
-		GtkStyle *style;
-		GdkColor color;
-		style = gtk_widget_get_style (GTK_WIDGET (location_bar->details->entry));
-		color = style->base[GTK_STATE_INSENSITIVE];
-		gtk_widget_modify_base(GTK_WIDGET (location_bar->details->entry), GTK_STATE_NORMAL, &color);
+		gtk_widget_override_background_color (GTK_WIDGET (location_bar->details->entry), GTK_STATE_FLAG_ACTIVE, NULL);
+	} else {
+		GtkStyleContext *style;
+		GdkRGBA color;
+
+		style = gtk_widget_get_style_context (GTK_WIDGET (location_bar->details->entry));
+		gtk_style_context_get_background_color (style, GTK_STATE_FLAG_INSENSITIVE, &color);
+		gtk_widget_override_background_color (GTK_WIDGET (location_bar->details->entry), GTK_STATE_FLAG_ACTIVE, &color);
 	}
 }
 
