@@ -33,6 +33,7 @@
 #include "nautilus-window-pane.h"
 #include "nautilus-spatial-window.h"
 #include "nautilus-navigation-window.h"
+#include "nautilus-bookmark-list.h"
 
 #include <libnautilus-private/nautilus-directory.h>
 
@@ -97,7 +98,8 @@ struct _NautilusNavigationWindowDetails {
         
         /* Side Pane */
         int side_pane_width;
-        NautilusSidebar *current_side_panel;
+        GtkWidget *sidebar;
+        gchar *sidebar_id;
 
 	/* Menus */
         GtkActionGroup *go_menu_action_group;
@@ -201,12 +203,7 @@ void                 nautilus_window_set_active_pane                     (Nautil
 NautilusWindowPane * nautilus_window_get_active_pane                     (NautilusWindow *window);
 
 void               nautilus_send_history_list_changed                    (void);
-void               nautilus_remove_from_history_list_no_notify           (GFile             *location);
 gboolean           nautilus_add_bookmark_to_history_list                 (NautilusBookmark  *bookmark);
-gboolean           nautilus_add_to_history_list_no_notify                (GFile             *location,
-									  const char        *name,
-									  gboolean           has_custom_name,
-									  GIcon            *icon);
 GList *            nautilus_get_history_list                             (void);
 void               nautilus_window_bookmarks_preference_changed_callback (gpointer           user_data);
 
